@@ -9,19 +9,12 @@
 #include "event.h"
 #include "shared.h"
 #include "folder.h"
+#include <libgen.h>
 
 #if defined (__APPLE__)
 #include "event.h"
 #endif
 
-int sort_by_stringptr (const void *p1, const void *p2)
-{
-  const char **s1 = (const char **) p1;
-  const char **s2 = (const char **) p2;
-
-  return strcmp (*s1, *s2);
-}
-
 static int get_exec_path (char *exec_path, const size_t exec_path_sz)
 {
   #if defined (__linux__) || defined (__CYGWIN__)
@@ -38,9 +31,9 @@ static int get_exec_path (char *exec_path, const size_t exec_path_sz)
 
   #elif defined (_WIN)
 
-  const DWORD len = GetModuleFileName (NULL, exec_path, exec_path_sz - 1);
+  memset (exec_path, 0, exec_path_sz);
 
-  if (len == 0) return -1;
+  const int len = 0;
 
   #elif defined (__APPLE__)
 
@@ -100,12 +93,43 @@ static void get_install_dir (char *install_dir, const char *exec_path)
 #if defined (_POSIX)
 static void get_profile_dir (char *profile_dir, const char *home_dir)
 {
-  snprintf (profile_dir, HCBUFSIZ_TINY - 1, "%s/%s", home_dir, DOT_HASHCAT);
+  snprintf (profile_dir, HCBUFSIZ_TINY, "%s/%s", home_dir, DOT_HASHCAT);
+
+  if (hc_path_is_directory (profile_dir)) return;
+
+  char *xdg_data_home = getenv ("XDG_DATA_HOME");
+
+  if (xdg_data_home)
+  {
+    snprintf (profile_dir, HCBUFSIZ_TINY, "%s/hashcat", xdg_data_home);
+  }
+  else
+  {
+    snprintf (profile_dir, HCBUFSIZ_TINY, "%s/.local/share/hashcat", home_dir);
+  }
+}
+
+static void get_cache_dir (char *cache_dir, const char *home_dir)
+{
+  snprintf (cache_dir, HCBUFSIZ_TINY, "%s/%s", home_dir, DOT_HASHCAT);
+
+  if (hc_path_is_directory (cache_dir)) return;
+
+  char *xdg_cache_home = getenv ("XDG_CACHE_HOME");
+
+  if (xdg_cache_home)
+  {
+    snprintf (cache_dir, HCBUFSIZ_TINY, "%s/hashcat", xdg_cache_home);
+  }
+  else
+  {
+    snprintf (cache_dir, HCBUFSIZ_TINY, "%s/.cache/hashcat", home_dir);
+  }
 }
 
 static void get_session_dir (char *session_dir, const char *profile_dir)
 {
-  snprintf (session_dir, HCBUFSIZ_TINY - 1, "%s/%s", profile_dir, SESSIONS_FOLDER);
+  snprintf (session_dir, HCBUFSIZ_TINY, "%s/%s", profile_dir, SESSIONS_FOLDER);
 }
 #endif
 
@@ -266,6 +290,7 @@ int folder_config_init (hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const char *ins
    * A workaround is to chdir() to the OpenCL folder,
    * then compile the kernels,
    * then chdir() back to where we came from so we need to save it first
+   * - temporary disabled due to https://github.com/hashcat/hashcat/issues/2379
    */
 
   char *cwd = (char *) hcmalloc (HCBUFSIZ_TINY);
@@ -329,7 +354,7 @@ int folder_config_init (hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const char *ins
 
   if (resolved_exec_path == NULL)
   {
-    event_log_error (hashcat_ctx, "%s: %s", resolved_exec_path, strerror (errno));
+    event_log_error (hashcat_ctx, "%s: %s", exec_path, strerror (errno));
 
     hcfree (cwd);
 
@@ -340,11 +365,12 @@ int folder_config_init (hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const char *ins
     return -1;
   }
 
-  char *install_dir = hcmalloc (HCBUFSIZ_TINY);
+  char *install_dir = (char *) hcmalloc (HCBUFSIZ_TINY);
 
   get_install_dir (install_dir, resolved_exec_path);
 
   char *profile_dir = NULL;
+  char *cache_dir   = NULL;
   char *session_dir = NULL;
   char *shared_dir  = NULL;
 
@@ -359,20 +385,24 @@ int folder_config_init (hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const char *ins
 
     const char *home_dir = pwp->pw_dir;
 
-    profile_dir = hcmalloc (HCBUFSIZ_TINY);
-    session_dir = hcmalloc (HCBUFSIZ_TINY);
+    profile_dir = (char *) hcmalloc (HCBUFSIZ_TINY);
+    cache_dir   = (char *) hcmalloc (HCBUFSIZ_TINY);
+    session_dir = (char *) hcmalloc (HCBUFSIZ_TINY);
 
     get_profile_dir (profile_dir, home_dir);
+    get_cache_dir   (cache_dir,   home_dir);
     get_session_dir (session_dir, profile_dir);
 
     shared_dir = hcstrdup (shared_folder);
 
-    hc_mkdir (profile_dir, 0700);
-    hc_mkdir (session_dir, 0700);
+    hc_mkdir_rec (profile_dir, 0700);
+    hc_mkdir_rec (cache_dir,   0700);
+    hc_mkdir     (session_dir, 0700);
   }
   else
   {
     profile_dir = install_dir;
+    cache_dir   = install_dir;
     session_dir = install_dir;
     shared_dir  = install_dir;
   }
@@ -387,6 +417,7 @@ int folder_config_init (hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const char *ins
   get_install_dir (install_dir, exec_path);
 
   char *profile_dir = install_dir;
+  char *cache_dir   = install_dir;
   char *session_dir = install_dir;
   char *shared_dir  = install_dir;
 
@@ -407,16 +438,9 @@ int folder_config_init (hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const char *ins
 
   hc_asprintf (&cpath, "%s\\OpenCL\\", shared_dir);
 
-  char *cpath_real = (char *) hcmalloc (HCBUFSIZ_TINY);
+  char *cpath_real;
 
-  if (GetFullPathName (cpath, HCBUFSIZ_TINY - 1, cpath_real, NULL) == 0)
-  {
-    event_log_error (hashcat_ctx, "%s: %s", cpath, "GetFullPathName().");
-
-    hcfree (cwd);
-
-    return -1;
-  }
+  hc_asprintf (&cpath_real, "%s\\OpenCL\\", shared_dir);
 
   #else
 
@@ -436,6 +460,7 @@ int folder_config_init (hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const char *ins
     // we prevent double-freeing the same memory address (this happens if e.g. profile_dir == session_dir)
 
     if (profile_dir == shared_dir) profile_dir = NULL;
+    if (cache_dir   == shared_dir) cache_dir   = NULL;
     if (session_dir == shared_dir) session_dir = NULL;
 
     shared_dir = NULL;
@@ -444,15 +469,21 @@ int folder_config_init (hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const char *ins
     hcfree (profile_dir);
 
     if (session_dir == profile_dir) session_dir = NULL;
+    if (cache_dir   == profile_dir) cache_dir   = NULL;
 
     profile_dir = NULL;
 
 
+    hcfree (cache_dir);
+
+    if (session_dir == cache_dir) session_dir = NULL;
+
+    cache_dir = NULL;
+
     hcfree (session_dir);
 
     session_dir = NULL;
 
-
     hcfree (cpath_real);
 
     cpath_real = NULL;
@@ -465,7 +496,8 @@ int folder_config_init (hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const char *ins
   hcfree (cpath);
 
   //if (getenv ("TMP") == NULL)
-  if (1)
+  /* temporary disabled due to https://github.com/hashcat/hashcat/issues/2379
+  if (true)
   {
     char *tmp;
 
@@ -473,6 +505,7 @@ int folder_config_init (hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const char *ins
 
     putenv (tmp);
   }
+  */
 
   #if defined (_WIN)
 
@@ -493,7 +526,7 @@ int folder_config_init (hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const char *ins
 
   char *kernels_folder;
 
-  hc_asprintf (&kernels_folder, "%s/kernels", profile_dir);
+  hc_asprintf (&kernels_folder, "%s/kernels", cache_dir);
 
   hc_mkdir (kernels_folder, 0700);
 
@@ -506,6 +539,7 @@ int folder_config_init (hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const char *ins
   folder_config->cwd          = cwd;
   folder_config->install_dir  = install_dir;
   folder_config->profile_dir  = profile_dir;
+  folder_config->cache_dir    = cache_dir;
   folder_config->session_dir  = session_dir;
   folder_config->shared_dir   = shared_dir;
   folder_config->cpath_real   = cpath_real;
@@ -532,3 +566,24 @@ int hc_mkdir (const char *name, MAYBE_UNUSED const int mode)
   return mkdir (name, mode);
   #endif
 }
+
+int hc_mkdir_rec (const char *path, MAYBE_UNUSED const int mode)
+{
+  char *fullpath = hcstrdup (path);
+
+  char *subpath = dirname (fullpath);
+
+  if (strlen (subpath) > 1)
+  {
+    if (hc_mkdir_rec (subpath, mode) == -1) return -1;
+  }
+
+  if (hc_mkdir (path, mode) == -1)
+  {
+    if (errno != EEXIST) return -1;
+  }
+
+  hcfree (fullpath);
+
+  return 0;
+}
