@@ -102,14 +102,7 @@ char *module_jit_build_options (MAYBE_UNUSED const hashconfig_t *hashconfig, MAY
 
   if (device_param->is_cuda == true)
   {
-    if (device_param->kernel_dynamic_local_mem_size_memset % device_param->device_local_mem_size)
-    {
-      // this is the case Compute Capability 7.5
-      // there is also Compute Capability 7.0 which offers a larger dynamic local size access
-      // however, if it's an exact multiple the driver can optimize this for us more efficient
-
-      use_dynamic = true;
-    }
+    use_dynamic = true;
   }
 
   // this uses some nice feedback effect.
