@@ -6,13 +6,11 @@
 #include "common.h"
 #include "types.h"
 #include "event.h"
-#include "memory.h"
 #include "hwmon.h"
 #include "timer.h"
 #include "hashes.h"
 #include "thread.h"
 #include "restore.h"
-#include "shared.h"
 #include "status.h"
 #include "monitor.h"
 
@@ -46,7 +44,7 @@ static int monitor (hashcat_ctx_t *hashcat_ctx)
 {
   hashes_t       *hashes        = hashcat_ctx->hashes;
   hwmon_ctx_t    *hwmon_ctx     = hashcat_ctx->hwmon_ctx;
-  opencl_ctx_t   *opencl_ctx    = hashcat_ctx->opencl_ctx;
+  backend_ctx_t  *backend_ctx   = hashcat_ctx->backend_ctx;
   restore_ctx_t  *restore_ctx   = hashcat_ctx->restore_ctx;
   status_ctx_t   *status_ctx    = hashcat_ctx->status_ctx;
   user_options_t *user_options  = hashcat_ctx->user_options;
@@ -72,7 +70,7 @@ static int monitor (hashcat_ctx_t *hashcat_ctx)
     restore_check = true;
   }
 
-  if ((user_options->remove == true) && (hashes->hashlist_mode == HL_MODE_FILE))
+  if ((user_options->remove == true) && ((hashes->hashlist_mode == HL_MODE_FILE_PLAIN) || (hashes->hashlist_mode == HL_MODE_FILE_BINARY)))
   {
     remove_check = true;
   }
@@ -112,17 +110,37 @@ static int monitor (hashcat_ctx_t *hashcat_ctx)
 
     if (status_ctx->devices_status == STATUS_INIT) continue;
 
-    if (hwmon_check == true)
+    if (hwmon_ctx->enabled == true)
     {
       hc_thread_mutex_lock (status_ctx->mux_hwmon);
 
-      for (u32 device_id = 0; device_id < opencl_ctx->devices_cnt; device_id++)
+      for (int backend_devices_idx = 0; backend_devices_idx < backend_ctx->backend_devices_cnt; backend_devices_idx++)
       {
-        hc_device_param_t *device_param = &opencl_ctx->devices_param[device_id];
+        hc_device_param_t *device_param = &backend_ctx->devices_param[backend_devices_idx];
 
         if (device_param->skipped == true) continue;
 
-        const int rc_throttle = hm_get_throttle_with_device_id (hashcat_ctx, device_id);
+        if ((backend_ctx->devices_param[backend_devices_idx].opencl_device_type & CL_DEVICE_TYPE_GPU) == 0) continue;
+
+        const int temperature = hm_get_temperature_with_devices_idx (hashcat_ctx, backend_devices_idx);
+
+        if (temperature > (int) user_options->hwmon_temp_abort)
+        {
+          EVENT_DATA (EVENT_MONITOR_TEMP_ABORT, &backend_devices_idx, sizeof (int));
+
+          myabort (hashcat_ctx);
+        }
+      }
+
+      for (int backend_devices_idx = 0; backend_devices_idx < backend_ctx->backend_devices_cnt; backend_devices_idx++)
+      {
+        hc_device_param_t *device_param = &backend_ctx->devices_param[backend_devices_idx];
+
+        if (device_param->skipped == true) continue;
+
+        if (device_param->skipped_warning == true) continue;
+
+        const int rc_throttle = hm_get_throttle_with_devices_idx (hashcat_ctx, backend_devices_idx);
 
         if (rc_throttle == -1) continue;
 
@@ -130,38 +148,13 @@ static int monitor (hashcat_ctx_t *hashcat_ctx)
         {
           slowdown_warnings++;
 
-          if (slowdown_warnings == 1) EVENT_DATA (EVENT_MONITOR_THROTTLE1, &device_id, sizeof (u32));
-          if (slowdown_warnings == 2) EVENT_DATA (EVENT_MONITOR_THROTTLE2, &device_id, sizeof (u32));
-          if (slowdown_warnings == 3) EVENT_DATA (EVENT_MONITOR_THROTTLE3, &device_id, sizeof (u32));
+          if (slowdown_warnings == 1) EVENT_DATA (EVENT_MONITOR_THROTTLE1, &backend_devices_idx, sizeof (int));
+          if (slowdown_warnings == 2) EVENT_DATA (EVENT_MONITOR_THROTTLE2, &backend_devices_idx, sizeof (int));
+          if (slowdown_warnings == 3) EVENT_DATA (EVENT_MONITOR_THROTTLE3, &backend_devices_idx, sizeof (int));
         }
         else
         {
-          slowdown_warnings = 0;
-        }
-      }
-
-      hc_thread_mutex_unlock (status_ctx->mux_hwmon);
-    }
-
-    if (hwmon_check == true && user_options->gpu_temp_disable == false)
-    {
-      hc_thread_mutex_lock (status_ctx->mux_hwmon);
-
-      for (u32 device_id = 0; device_id < opencl_ctx->devices_cnt; device_id++)
-      {
-        hc_device_param_t *device_param = &opencl_ctx->devices_param[device_id];
-
-        if (device_param->skipped == true) continue;
-
-        if ((opencl_ctx->devices_param[device_id].device_type & CL_DEVICE_TYPE_GPU) == 0) continue;
-
-        const int temperature = hm_get_temperature_with_device_id (hashcat_ctx, device_id);
-
-        if (temperature > (int) user_options->gpu_temp_abort)
-        {
-          EVENT_DATA (EVENT_MONITOR_TEMP_ABORT, &device_id, sizeof (u32));
-
-          myabort (hashcat_ctx);
+          if (slowdown_warnings > 0) slowdown_warnings--;
         }
       }
 
@@ -174,9 +167,14 @@ static int monitor (hashcat_ctx_t *hashcat_ctx)
 
       if (restore_left == 0)
       {
-        const int rc = cycle_restore (hashcat_ctx);
+        // Can't return from monitor for that reasons, see:
+        // https://github.com/hashcat/hashcat/issues/2704
+        //
+        //const int rc = cycle_restore (hashcat_ctx);
+        //
+        //if (rc == -1) return -1;
 
-        if (rc == -1) return -1;
+        cycle_restore (hashcat_ctx);
 
         restore_left = user_options->restore_timer;
       }
@@ -204,9 +202,14 @@ static int monitor (hashcat_ctx_t *hashcat_ctx)
         {
           hashes->digests_saved = hashes->digests_done;
 
-          const int rc = save_hash (hashcat_ctx);
+          // Can't return from monitor for that reasons, see:
+          // https://github.com/hashcat/hashcat/issues/2704
+          //
+          // const int rc = save_hash (hashcat_ctx);
+          //
+          // if (rc == -1) return -1;
 
-          if (rc == -1) return -1;
+          save_hash (hashcat_ctx);
         }
 
         remove_left = user_options->remove_timer;
@@ -239,19 +242,21 @@ static int monitor (hashcat_ctx_t *hashcat_ctx)
 
       hc_thread_mutex_lock (status_ctx->mux_hwmon);
 
-      for (u32 device_id = 0; device_id < opencl_ctx->devices_cnt; device_id++)
+      for (int backend_devices_idx = 0; backend_devices_idx < backend_ctx->backend_devices_cnt; backend_devices_idx++)
       {
-        hc_device_param_t *device_param = &opencl_ctx->devices_param[device_id];
+        hc_device_param_t *device_param = &backend_ctx->devices_param[backend_devices_idx];
 
         if (device_param->skipped == true) continue;
 
+        if (device_param->skipped_warning == true) continue;
+
         exec_cnt++;
 
-        const double exec = status_get_exec_msec_dev (hashcat_ctx, device_id);
+        const double exec = status_get_exec_msec_dev (hashcat_ctx, backend_devices_idx);
 
         exec_total += exec;
 
-        const int util = hm_get_utilization_with_device_id (hashcat_ctx, device_id);
+        const int util = hm_get_utilization_with_devices_idx (hashcat_ctx, backend_devices_idx);
 
         if (util == -1) continue;
 
@@ -284,25 +289,29 @@ static int monitor (hashcat_ctx_t *hashcat_ctx)
     }
 
     // stdin read timeout check
+    // note: we skip the stdin timeout check if it was disabled with stdin_timeout_abort set to 0
 
-    if (status_get_progress_done (hashcat_ctx) == 0)
+    if (user_options->stdin_timeout_abort != 0)
     {
-      if (status_ctx->stdin_read_timeout_cnt >= STDIN_TIMEOUT_MIN)
+      if (status_get_progress_done (hashcat_ctx) == 0)
       {
-        if (status_ctx->stdin_read_timeout_cnt >= STDIN_TIMEOUT_MAX)
+        if (status_ctx->stdin_read_timeout_cnt > 0)
         {
-          EVENT_DATA (EVENT_MONITOR_NOINPUT_ABORT, NULL, 0);
+          if (status_ctx->stdin_read_timeout_cnt >= user_options->stdin_timeout_abort)
+          {
+            EVENT_DATA (EVENT_MONITOR_NOINPUT_ABORT, NULL, 0);
 
-          myabort (hashcat_ctx);
+            myabort (hashcat_ctx);
 
-          status_ctx->shutdown_inner = true;
+            status_ctx->shutdown_inner = true;
 
-          break;
-        }
+            break;
+          }
 
-        if ((status_ctx->stdin_read_timeout_cnt % STDIN_TIMEOUT_MIN) == 0)
-        {
-          EVENT_DATA (EVENT_MONITOR_NOINPUT_HINT, NULL, 0);
+          if ((status_ctx->stdin_read_timeout_cnt % STDIN_TIMEOUT_WARN) == 0)
+          {
+            EVENT_DATA (EVENT_MONITOR_NOINPUT_HINT, NULL, 0);
+          }
         }
       }
     }
@@ -314,9 +323,14 @@ static int monitor (hashcat_ctx_t *hashcat_ctx)
   {
     if (hashes->digests_saved != hashes->digests_done)
     {
-      const int rc = save_hash (hashcat_ctx);
+      // Can't return from monitor for that reasons, see:
+      // https://github.com/hashcat/hashcat/issues/2704
+      //
+      // const int rc = save_hash (hashcat_ctx);
+      //
+      // if (rc == -1) return -1;
 
-      if (rc == -1) return -1;
+      save_hash (hashcat_ctx);
     }
   }
 
@@ -324,9 +338,14 @@ static int monitor (hashcat_ctx_t *hashcat_ctx)
 
   if (restore_check == true)
   {
-    const int rc = cycle_restore (hashcat_ctx);
+    // Can't return from monitor for that reasons, see:
+    // https://github.com/hashcat/hashcat/issues/2704
+    //
+    // const int rc = cycle_restore (hashcat_ctx);
+    //
+    // if (rc == -1) return -1;
 
-    if (rc == -1) return -1;
+    cycle_restore (hashcat_ctx);
   }
 
   return 0;
