@@ -30,6 +30,7 @@ void welcome_screen (hashcat_ctx_t *hashcat_ctx, const char *version_tag)
   if (user_options->stdout_flag == true) return;
   if (user_options->show        == true) return;
   if (user_options->left        == true) return;
+  if (user_options->identify    == true) return;
 
   if (user_options->benchmark == true)
   {
@@ -68,6 +69,16 @@ void welcome_screen (hashcat_ctx_t *hashcat_ctx, const char *version_tag)
     event_log_info (hashcat_ctx, "%s (%s) starting in progress-only mode...", PROGNAME, version_tag);
     event_log_info (hashcat_ctx, NULL);
   }
+  else if (user_options->backend_info == true)
+  {
+    event_log_info (hashcat_ctx, "%s (%s) starting in backend information mode...", PROGNAME, version_tag);
+    event_log_info (hashcat_ctx, NULL);
+  }
+  else if (user_options->hash_mode_chgd == false)
+  {
+    event_log_info (hashcat_ctx, "%s (%s) starting in autodetect mode...", PROGNAME, version_tag);
+    event_log_info (hashcat_ctx, NULL);
+  }
   else
   {
     event_log_info (hashcat_ctx, "%s (%s) starting...", PROGNAME, version_tag);
@@ -91,6 +102,7 @@ void goodbye_screen (hashcat_ctx_t *hashcat_ctx, const time_t proc_start, const
   if (user_options->stdout_flag == true) return;
   if (user_options->show        == true) return;
   if (user_options->left        == true) return;
+  if (user_options->identify    == true) return;
 
   char start_buf[32]; memset (start_buf, 0, sizeof (start_buf));
   char stop_buf[32];  memset (start_buf, 0, sizeof (stop_buf));
@@ -1237,6 +1249,15 @@ void status_display (hashcat_ctx_t *hashcat_ctx)
     hashcat_status->time_estimated_relative);
   }
 
+  if (hashconfig->opti_type & OPTI_TYPE_OPTIMIZED_KERNEL)
+  {
+    event_log_info (hashcat_ctx, "Kernel.Feature...: Optimized Kernel");
+  }
+  else
+  {
+    event_log_info (hashcat_ctx, "Kernel.Feature...: Pure Kernel");
+  }
+
   switch (hashcat_status->guess_mode)
   {
     case GUESS_MODE_STRAIGHT_FILE:
@@ -1733,6 +1754,16 @@ void status_display (hashcat_ctx_t *hashcat_ctx)
       device_info->iteration_pos_dev + device_info->iteration_left_dev);
   }
 
+  //if (hashconfig->opts_type & OPTS_TYPE_SLOW_CANDIDATES)
+  if (user_options->slow_candidates == true)
+  {
+    event_log_info (hashcat_ctx, "Candidate.Engine.: Host Generator + PCIe");
+  }
+  else
+  {
+    event_log_info (hashcat_ctx, "Candidate.Engine.: Device Generator");
+  }
+
   for (int device_id = 0; device_id < hashcat_status->device_info_cnt; device_id++)
   {
     const device_info_t *device_info = hashcat_status->device_info_buf + device_id;
